"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
const Fluentd = require("fluent-logger");
const os = require("os");
const util_1 = require("util");
/**
 * Clase para dar soporte al manejo de los hilos
 * sobre los microservicios.
 * @class
 */
class FluentdLogs {
    constructor() {
        /**
         *  Logger de fluentd
         */
        this.Fluentd = null;
        /**
         *  Configuracion del servicio
         */
        this.confService = null;
        // Generar el sender de fluent
        this.Fluentd = Fluentd.createFluentSender('microservices.logs', {
            host: process.env.FLUENTD_HOST
                ? process.env.FLUENTD_HOST
                : 'td-agent',
            port: process.env.FLUENTD_PORT
                ? process.env.FLUENTD_PORT
                : 24224,
            timeout: 3.0,
            reconnectInterval: 600000,
        });
    }
    /**
     * Metodo para iniciar la configuracion de los datos del
     * logs controler.
     * @param conf Configuracion del servicio
     */
    config(conf) {
        this.confService = conf;
    }
    /**
     * Info
     * @param message Mensaje a publicar
     */
    info(message, ...data) {
        this.publishLog(types_1.TypeLogs.Info, message, null, data);
    }
    /**
     * Warning
     * @param message Mensaje a publicar
     */
    warning(message, ...data) {
        this.publishLog(types_1.TypeLogs.Warning, message, null, data);
    }
    /**
     * Error
     * @param message Mensaje a publicar
     */
    error(message, error, ...data) {
        this.publishLog(types_1.TypeLogs.Error, message, error, data);
    }
    /**
     * Log
     * @param message Mensaje a publicar
     */
    log(message, ...data) {
        this.publishLog(types_1.TypeLogs.Log, message, null, data);
    }
    /**
     *  Metodo para enviar el mensaje de logs
     * @param Type Tipo del log
     * @param Message Mensaje a enviar
     * @param error Objeto con el error
     * @param rest Otros parametros
     */
    publishLog(Type, Message, error, ...rest) {
        // Mensaje original
        const formatLog = {
            'schema-version': '1.0',
            level: Type,
            'error-origin': error ? 'INTERNAL' : null,
            tracing_id: null,
            dockerID: null,
            hostname: os.hostname(),
            microservice: `${this.confService.App}_${this.confService.Version}_${this.confService.Name}`,
            message: Message,
            operation: null,
            object: null,
            stacktrace: null,
            timestamp: new Date().toISOString(),
        };
        // Validar los objetos extra
        rest.forEach((value) => {
            if (util_1.isObject(value)) {
                formatLog.object = Object.assign(util_1.isObject(formatLog.object) ? formatLog.object : {}, value);
            }
        });
        // Validar el error
        if (error) {
            formatLog.stacktrace = error;
        }
        // Enviar el error
        this.Fluentd.emit('app', formatLog);
    }
}
exports.default = FluentdLogs;
